from typing import Union

Value = Union[int, float]


class Calculator:
    def __init__(self):
        self.memory_value = 0

    @staticmethod
    def validate_value(value):
        if not isinstance(value, (int, float)):
            raise TypeError("Input must be numeric!")

    def add(self, value: Value) -> Value:
        """Add value to memory.

            Example:
            >>> calc = Calculator()
            >>> calc.add(5)
            5
            >>> calc.add(1.5)
            6.5
            >>> calc.reset_memory()

        """
        self.validate_value(value)
        self.memory_value += value
        return self.memory_value

    def subtract(self, value: Value) -> Value:
        """Subtract value from the memory.

            Example:
            >>> calc = Calculator()
            >>> calc.subtract(5)
            -5
            >>> calc.subtract(1.5)
            -6.5
            >>> calc.reset_memory()

        """

        self.validate_value(value)
        self.memory_value -= value
        return self.memory_value

    def multiply(self, value: Value) -> Value:
        """Multiply memory value by value

            Example:
            >>> calc = Calculator()
            >>> calc.add(5)
            5
            >>> calc.multiply(5)
            25
            >>> calc.multiply(-0.5)
            -12.5
            >>> calc.reset_memory()

        """

        self.validate_value(value)
        self.memory_value *= value
        return self.memory_value

    def divide(self, value: Value) -> Value:
        """Divide memory value by value

            Example:
            >>> calc = Calculator()
            >>> calc.add(5)
            5
            >>> calc.divide(5)
            1.0
            >>> calc.divide(-0.5)
            -2.0
            >>> calc.reset_memory()

        """

        self.validate_value(value)
        if value != 0.0:
            self.memory_value /= value
            return self.memory_value
        else:
            raise ValueError(f"Cannot divide by zero!")

    def n_root(self, value: Value, n_root: Value) -> Value:
        """Divide memory value by value

            Example:
            >>> calc = Calculator()
            >>> calc.n_root(25, 2)
            5.0
            >>> calc.n_root(0.125, 3)
            0.5
            >>> calc.reset_memory()


        """
        self.validate_value(value)
        self.validate_value(n_root)
        if n_root == 0:
            raise ValueError(f"Root by 0 is undefined!")
        if value >= 0.0:
            self.memory_value = value ** (1.0 / n_root)
            return self.memory_value
        elif n_root % 2 == 0 and value < 0.0:
            raise ValueError(f"Cannot calculate even root"
                             f" of a negative number")
        else:
            value *= -1
            self.memory_value = -1 * value ** (1.0 / n_root)
            return self.memory_value



    def reset_memory(self):
        self.memory_value = 0



